@@ -2,10 +2,9 @@
 Makefile options for stanc and C++ compilers
 """
 
-import logging
 import os
 from pathlib import Path
-from typing import Dict, List
+from typing import Any, Dict, List, Optional, Union
 
 from cmdstanpy.utils import get_logger
 
@@ -16,6 +15,7 @@ STANC_OPTS = [
     'warn-uninitialized',
     'include_paths',
     'name',
+    'warn-pedantic',
 ]
 
 STANC_IGNORE_OPTS = [
@@ -39,14 +39,6 @@ STANC_IGNORE_OPTS = [
     'version',
 ]
 
-CPP_OPTS = [
-    'STAN_OPENCL',
-    'OPENCL_DEVICE_ID',
-    'OPENCL_PLATFORM_ID',
-    'STAN_MPI',
-    'STAN_THREADS',
-]
-
 
 class CompilerOptions:
     """
@@ -55,34 +47,61 @@ class CompilerOptions:
     Attributes:
         stanc_options - stanc compiler flags, options
         cpp_options - makefile options (NAME=value)
+        user_header - path to a user .hpp file to include during compilation
     """
 
     def __init__(
         self,
-        stanc_options: Dict = None,
-        cpp_options: Dict = None,
-        logger: logging.Logger = None,
+        *,
+        stanc_options: Optional[Dict[str, Any]] = None,
+        cpp_options: Optional[Dict[str, Any]] = None,
+        user_header: Optional[str] = None,
     ) -> None:
         """Initialize object."""
-        self._stanc_options = stanc_options
-        self._cpp_options = cpp_options
-        self._logger = logger or get_logger()
+        self._stanc_options = stanc_options if stanc_options is not None else {}
+        self._cpp_options = cpp_options if cpp_options is not None else {}
+        self._user_header = user_header if user_header is not None else ''
 
     def __repr__(self) -> str:
         return 'stanc_options={}, cpp_options={}'.format(
             self._stanc_options, self._cpp_options
         )
 
+    def __eq__(self, other: Any) -> bool:
+        """Overrides the default implementation"""
+        if self.is_empty() and other is None:  # equiv w/r/t compiler
+            return True
+        if not isinstance(other, CompilerOptions):
+            return False
+        return (
+            self._stanc_options == other.stanc_options
+            and self._cpp_options == other.cpp_options
+            and self._user_header == other.user_header
+        )
+
+    def is_empty(self) -> bool:
+        """True if no options specified."""
+        return (
+            self._stanc_options == {}
+            and self._cpp_options == {}
+            and self._user_header == ''
+        )
+
     @property
-    def stanc_options(self) -> Dict:
+    def stanc_options(self) -> Dict[str, Union[bool, int, str]]:
         """Stanc compiler options."""
         return self._stanc_options
 
     @property
-    def cpp_options(self) -> Dict:
+    def cpp_options(self) -> Dict[str, Union[bool, int]]:
         """C++ compiler options."""
         return self._cpp_options
 
+    @property
+    def user_header(self) -> str:
+        """user header."""
+        return self._user_header
+
     def validate(self) -> None:
         """
         Check compiler args.
@@ -90,6 +109,7 @@ class CompilerOptions:
         """
         self.validate_stanc_opts()
         self.validate_cpp_opts()
+        self.validate_user_header()
 
     def validate_stanc_opts(self) -> None:
         """
@@ -103,12 +123,10 @@ class CompilerOptions:
         paths = None
         for key, val in self._stanc_options.items():
             if key in STANC_IGNORE_OPTS:
-                self._logger.info('ignoring compiler option: %s', key)
+                get_logger().info('ignoring compiler option: %s', key)
                 ignore.append(key)
             elif key not in STANC_OPTS:
-                raise ValueError(
-                    'unknown stanc compiler option: {}'.format(key)
-                )
+                raise ValueError(f'unknown stanc compiler option: {key}')
             elif key == 'include_paths':
                 paths = val
                 if isinstance(val, str):
@@ -116,7 +134,7 @@ class CompilerOptions:
                 elif not isinstance(val, list):
                     raise ValueError(
                         'Invalid include_paths, expecting list or '
-                        'string, found type: {}.'.format(type(val))
+                        f'string, found type: {type(val)}.'
                     )
             elif key == 'use-opencl':
                 if self._cpp_options is None:
@@ -145,24 +163,54 @@ class CompilerOptions:
         """
         if self._cpp_options is None:
             return
-        if (
-            'OPENCL_DEVICE_ID' in self._cpp_options.keys()
-            or 'OPENCL_PLATFORM_ID' in self._cpp_options.keys()
-        ):
-            self._cpp_options['STAN_OPENCL'] = 'TRUE'
-
-        for key, val in self._cpp_options.items():
-            if key not in CPP_OPTS:
-                raise ValueError(
-                    'unknown CmdStan makefile option: {}'.format(key)
-                )
-            if key in ['OPENCL_DEVICE_ID', 'OPENCL_PLATFORM_ID']:
+        for key in ['OPENCL_DEVICE_ID', 'OPENCL_PLATFORM_ID']:
+            if key in self._cpp_options:
+                self._cpp_options['STAN_OPENCL'] = 'TRUE'
+                val = self._cpp_options[key]
                 if not isinstance(val, int) or val < 0:
                     raise ValueError(
-                        '{} must be a non-negative integer value,'
-                        ' found {}.'.format(key, val)
+                        f'{key} must be a non-negative integer value,'
+                        f' found {val}.'
                     )
 
+    def validate_user_header(self) -> None:
+        """
+        User header exists.
+        Raise ValueError if bad config is found.
+        """
+        if self._user_header != "":
+            if not (
+                os.path.exists(self._user_header)
+                and os.path.isfile(self._user_header)
+            ):
+                raise ValueError(
+                    f"User header file {self._user_header} cannot be found"
+                )
+            if self._user_header[-4:] != '.hpp':
+                raise ValueError(
+                    f"Header file must end in .hpp, got {self._user_header}"
+                )
+            if "allow_undefined" not in self._stanc_options:
+                self._stanc_options["allow_undefined"] = True
+            # set full path
+            self._user_header = os.path.abspath(self._user_header)
+
+            if ' ' in self._user_header:
+                raise ValueError(
+                    "User header must be in a location with no spaces in path!"
+                )
+
+            if (
+                'USER_HEADER' in self._cpp_options
+                and self._user_header != self._cpp_options['USER_HEADER']
+            ):
+                raise ValueError(
+                    "Disagreement in user_header C++ options found!\n"
+                    f"{self._user_header}, {self._cpp_options['USER_HEADER']}"
+                )
+
+            self._cpp_options['USER_HEADER'] = self._user_header
+
     def add(self, new_opts: "CompilerOptions") -> None:  # noqa: disable=Q000
         """Adds options to existing set of compiler options."""
         if new_opts.stanc_options is not None:
@@ -177,6 +225,8 @@ class CompilerOptions:
         if new_opts.cpp_options is not None:
             for key, val in new_opts.cpp_options.items():
                 self._cpp_options[key] = val
+        if new_opts._user_header != '' and self._user_header == '':
+            self._user_header = new_opts._user_header
 
     def add_include_path(self, path: str) -> None:
         """Adds include path to existing set of compiler options."""
@@ -201,10 +251,10 @@ class CompilerOptions:
                         )
                     )
                 elif key == 'name':
-                    opts.append('STANCFLAGS+=--{}={}'.format(key, val))
+                    opts.append(f'STANCFLAGS+=--name={val}')
                 else:
-                    opts.append('STANCFLAGS+=--{}'.format(key))
+                    opts.append(f'STANCFLAGS+=--{key}')
         if self._cpp_options is not None and len(self._cpp_options) > 0:
             for key, val in self._cpp_options.items():
-                opts.append('{}={}'.format(key, val))
+                opts.append(f'{key}={val}')
         return opts
