@@ -1,18 +1,16 @@
 """CmdStanModel tests"""
 
+import logging
 import os
 import shutil
 import tempfile
 import unittest
-from unittest.mock import Mock
+from test import CustomTestCase
 
-import numpy as np
-import pytest
-import tqdm
-from testfixtures import LogCapture
+from testfixtures import LogCapture, StringComparison
 
 from cmdstanpy.model import CmdStanModel
-from cmdstanpy.utils import EXTENSION, cmdstan_path
+from cmdstanpy.utils import EXTENSION
 
 HERE = os.path.dirname(os.path.abspath(__file__))
 DATAFILES_PATH = os.path.join(HERE, 'data')
@@ -31,33 +29,25 @@ model {
 """
 
 BERN_STAN = os.path.join(DATAFILES_PATH, 'bernoulli.stan')
+BERN_DATA = os.path.join(DATAFILES_PATH, 'bernoulli.data.json')
 BERN_EXE = os.path.join(DATAFILES_PATH, 'bernoulli' + EXTENSION)
 BERN_BASENAME = 'bernoulli'
 
 
-class CmdStanModelTest(unittest.TestCase):
-
-    # pylint: disable=no-self-use
-    @pytest.fixture(scope='class', autouse=True)
-    def do_clean_up(self):
-        for root, _, files in os.walk(DATAFILES_PATH):
-            for filename in files:
-                _, ext = os.path.splitext(filename)
-                if ext.lower() in ('.o', '.d', '.hpp', '.exe', ''):
-                    filepath = os.path.join(root, filename)
-                    os.remove(filepath)
-
-    def show_cmdstan_version(self):
-        print('\n\nCmdStan version: {}\n\n'.format(cmdstan_path()))
-        self.assertTrue(True)
-
+class CmdStanModelTest(CustomTestCase):
     def test_model_good(self):
         # compile on instantiation, override model name
         model = CmdStanModel(model_name='bern', stan_file=BERN_STAN)
         self.assertEqual(BERN_STAN, model.stan_file)
-        self.assertTrue(model.exe_file.endswith(BERN_EXE.replace('\\', '/')))
+        self.assertPathsEqual(model.exe_file, BERN_EXE)
         self.assertEqual('bern', model.name)
 
+        # compile with external header
+        model = CmdStanModel(
+            stan_file=os.path.join(DATAFILES_PATH, "external.stan"),
+            user_header=os.path.join(DATAFILES_PATH, 'return_one.hpp'),
+        )
+
         # default model name
         model = CmdStanModel(stan_file=BERN_STAN)
         self.assertEqual(BERN_BASENAME, model.name)
@@ -65,21 +55,58 @@ class CmdStanModelTest(unittest.TestCase):
         # instantiate with existing exe
         model = CmdStanModel(stan_file=BERN_STAN, exe_file=BERN_EXE)
         self.assertEqual(BERN_STAN, model.stan_file)
-        self.assertTrue(model.exe_file.endswith(BERN_EXE))
+        self.assertPathsEqual(model.exe_file, BERN_EXE)
 
-        # instantiate with existing exe only - no model
-        model2 = CmdStanModel(exe_file=BERN_EXE)
-        self.assertEqual(BERN_EXE, model2.exe_file)
+    def test_ctor_compile_arg(self):
+        # instantiate, don't compile
+        if os.path.exists(BERN_EXE):
+            os.remove(BERN_EXE)
+        model = CmdStanModel(stan_file=BERN_STAN, compile=False)
+        self.assertEqual(BERN_STAN, model.stan_file)
+        self.assertEqual(None, model.exe_file)
+
+        model = CmdStanModel(stan_file=BERN_STAN, compile=True)
+        self.assertPathsEqual(model.exe_file, BERN_EXE)
+        exe_time = os.path.getmtime(model.exe_file)
+
+        model = CmdStanModel(stan_file=BERN_STAN)
+        self.assertTrue(exe_time == os.path.getmtime(model.exe_file))
+
+        model = CmdStanModel(stan_file=BERN_STAN, compile='force')
+        self.assertTrue(exe_time < os.path.getmtime(model.exe_file))
+
+    def test_exe_only(self):
+        model = CmdStanModel(stan_file=BERN_STAN)
+        self.assertEqual(BERN_EXE, model.exe_file)
+        exe_only = os.path.join(DATAFILES_PATH, 'exe_only')
+        shutil.copyfile(model.exe_file, exe_only)
+
+        model2 = CmdStanModel(exe_file=exe_only)
         with self.assertRaises(RuntimeError):
             model2.code()
         with self.assertRaises(RuntimeError):
             model2.compile()
+        self.assertFalse(model2._fixed_param)
 
-        # instantiate, don't compile
-        os.remove(BERN_EXE)
-        model = CmdStanModel(stan_file=BERN_STAN, compile=False)
-        self.assertEqual(BERN_STAN, model.stan_file)
-        self.assertEqual(None, model.exe_file)
+    def test_fixed_param(self):
+        stan = os.path.join(DATAFILES_PATH, 'datagen_poisson_glm.stan')
+        model = CmdStanModel(stan_file=stan)
+        self.assertTrue(model._fixed_param)
+
+    # pylint: disable=no-self-use
+    def test_model_pedantic(self):
+        stan_file = os.path.join(DATAFILES_PATH, 'bernoulli_pedantic.stan')
+        with LogCapture(level=logging.WARNING) as log:
+            logging.getLogger()
+            model = CmdStanModel(model_name='bern', stan_file=stan_file)
+            model.compile(force=True, stanc_options={'warn-pedantic': True})
+        log.check_present(
+            (
+                'cmdstanpy',
+                'WARNING',
+                StringComparison(r'(?s).*The parameter theta has no priors.*'),
+            )
+        )
 
     def test_model_bad(self):
         with self.assertRaises(ValueError):
@@ -90,6 +117,14 @@ class CmdStanModelTest(unittest.TestCase):
             CmdStanModel(model_name='', stan_file=BERN_STAN)
         with self.assertRaises(ValueError):
             CmdStanModel(model_name='   ', stan_file=BERN_STAN)
+        with self.assertRaises(ValueError):
+            CmdStanModel(
+                stan_file=os.path.join(DATAFILES_PATH, "external.stan")
+            )
+        CmdStanModel(stan_file=BERN_STAN)
+        os.remove(BERN_EXE)
+        with self.assertRaises(ValueError):
+            CmdStanModel(stan_file=BERN_STAN, exe_file=BERN_EXE)
 
     def test_stanc_options(self):
         opts = {
@@ -140,6 +175,65 @@ class CmdStanModelTest(unittest.TestCase):
         self.assertEqual(cpp_opts['STAN_MPI'], 'TRUE')
         self.assertEqual(cpp_opts['STAN_THREADS'], 'TRUE')
 
+    def test_model_info(self):
+        model = CmdStanModel(stan_file=BERN_STAN, compile=False)
+        model.compile(force=True)
+        info_dict = model.exe_info()
+        self.assertEqual(info_dict['STAN_THREADS'].lower(), 'false')
+
+        if model.exe_file is not None and os.path.exists(model.exe_file):
+            os.remove(model.exe_file)
+        empty_dict = model.exe_info()
+        self.assertEqual(len(empty_dict), 0)
+
+        model_info = model.src_info()
+        self.assertNotEqual(model_info, {})
+        self.assertIn('theta', model_info['parameters'])
+
+    def test_compile_force(self):
+        if os.path.exists(BERN_EXE):
+            os.remove(BERN_EXE)
+        model = CmdStanModel(stan_file=BERN_STAN, compile=False, cpp_options={})
+        self.assertIsNone(model.exe_file)
+
+        model.compile(force=True)
+        self.assertIsNotNone(model.exe_file)
+        self.assertTrue(os.path.exists(model.exe_file))
+
+        info_dict = model.exe_info()
+        print(f'info={info_dict}')
+        self.assertEqual(info_dict['STAN_THREADS'].lower(), 'false')
+
+        more_opts = {'STAN_THREADS': 'TRUE'}
+
+        model.compile(force=True, cpp_options=more_opts)
+        self.assertIsNotNone(model.exe_file)
+        self.assertTrue(os.path.exists(model.exe_file))
+
+        info_dict2 = model.exe_info()
+        print(f'info2={info_dict2}')
+        self.assertEqual(info_dict2['STAN_THREADS'].lower(), 'true')
+
+        override_opts = {'STAN_NO_RANGE_CHECKS': 'TRUE'}
+
+        model.compile(
+            force=True, cpp_options=override_opts, override_options=True
+        )
+        info_dict3 = model.exe_info()
+        print(f'info3={info_dict3}')
+        self.assertEqual(info_dict3['STAN_THREADS'].lower(), 'false')
+        # cmdstan#1056
+        # self.assertEqual(info_dict3['STAN_NO_RANGE_CHECKS'].lower(), 'true')
+
+        model.compile(force=True, cpp_options=more_opts)
+        info_dict4 = model.exe_info()
+        self.assertEqual(info_dict4['STAN_THREADS'].lower(), 'true')
+
+        # test compile='force' in constructor
+        model2 = CmdStanModel(stan_file=BERN_STAN, compile='force')
+        info_dict5 = model2.exe_info()
+        self.assertEqual(info_dict5['STAN_THREADS'].lower(), 'false')
+
     def test_model_paths(self):
         # pylint: disable=unused-variable
         model = CmdStanModel(stan_file=BERN_STAN)  # instantiates exe
@@ -191,16 +285,13 @@ class CmdStanModelTest(unittest.TestCase):
 
     def test_model_syntax_error(self):
         stan = os.path.join(DATAFILES_PATH, 'bad_syntax.stan')
-        with LogCapture() as log:
-            with self.assertRaises(Exception):
+        with LogCapture(level=logging.WARNING) as log:
+            logging.getLogger()
+            with self.assertRaises(ValueError):
                 CmdStanModel(stan_file=stan)
-
-            # Join all the log messages into one string
-            error_message = '@( * O * )@'.join(np.array(log.actual())[:, -1])
-
-            # Ensure the new line character in error message is not escaped
-            # so the error message is readable
-            self.assertRegex(error_message, r'parsing error:(\r\n|\r|\n)')
+        log.check_present(
+            ('cmdstanpy', 'WARNING', StringComparison(r'(?s).*Syntax error.*'))
+        )
 
     def test_repr(self):
         model = CmdStanModel(stan_file=BERN_STAN)
@@ -213,10 +304,10 @@ class CmdStanModelTest(unittest.TestCase):
 
     def test_model_compile(self):
         model = CmdStanModel(stan_file=BERN_STAN)
-        self.assertTrue(model.exe_file.endswith(BERN_EXE.replace('\\', '/')))
+        self.assertPathsEqual(model.exe_file, BERN_EXE)
 
         model = CmdStanModel(stan_file=BERN_STAN)
-        self.assertTrue(model.exe_file.endswith(BERN_EXE.replace('\\', '/')))
+        self.assertPathsEqual(model.exe_file, BERN_EXE)
         old_exe_time = os.path.getmtime(model.exe_file)
         os.remove(BERN_EXE)
         model.compile()
@@ -261,7 +352,7 @@ class CmdStanModelTest(unittest.TestCase):
             stan_file=BERN_STAN, stanc_options={'include_paths': DATAFILES_PATH}
         )
         self.assertEqual(BERN_STAN, model.stan_file)
-        self.assertTrue(model.exe_file.endswith(BERN_EXE.replace('\\', '/')))
+        self.assertPathsEqual(model.exe_file, BERN_EXE)
 
     def test_model_includes_implicit(self):
         stan = os.path.join(DATAFILES_PATH, 'bernoulli_include.stan')
@@ -269,38 +360,7 @@ class CmdStanModelTest(unittest.TestCase):
         if os.path.exists(exe):
             os.remove(exe)
         model2 = CmdStanModel(stan_file=stan)
-        self.assertTrue(model2.exe_file.endswith(exe.replace('\\', '/')))
-
-    def test_read_progress(self):
-        model = CmdStanModel(stan_file=BERN_STAN, compile=False)
-
-        proc_mock = Mock()
-        proc_mock.poll.side_effect = [None, None, 'finish']
-        stan_output1 = 'Iteration: 12100 / 31000 [ 39%]  (Warmup)'
-        stan_output2 = 'Iteration: 14000 / 31000 [ 45%]  (Warmup)'
-        pbar = tqdm.tqdm(desc='Chain 1 - warmup', position=1, total=1)
-
-        proc_mock.stdout.readline.side_effect = [
-            stan_output1.encode('utf-8'),
-            stan_output2.encode('utf-8'),
-        ]
-
-        with LogCapture() as log:
-            result = model._read_progress(proc=proc_mock, pbar=pbar, idx=0)
-            self.assertEqual([], log.actual())
-            self.assertEqual(31000, pbar.total)
-
-            # Expect progress bar output to be something like this:
-            # 'Chain 1 -   done:  45%|████▌     | 14000/31000'
-            # --------
-
-            self.assertIn('Chain 1 -   done:  45%', str(pbar))
-            self.assertIn('14000/31000', str(pbar))
-
-            # Check Stan's output is returned
-            output = result.decode('utf-8')
-            self.assertIn(stan_output1, output)
-            self.assertIn(stan_output2, output)
+        self.assertPathsEqual(model2.exe_file, exe)
 
 
 if __name__ == '__main__':
